{Win32}

Unit WWave32;

Interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls;

Type
  TWhereWave = (wwInFile, wwInExe);
  TWaveOption = (woSync, woNoDefault, woLoop, woNoStop);
  TWaveOptions = set of TwaveOption;

  TWave = class(TComponent)
    private
      FWhereWave : TWhereWave;
      FWaveName : PChar;
      FResourceType : PChar;
      FOptions : word;
      FPalyOptions : TWaveOptions;
      procedure SetWhereWave(Value : TWhereWave);
      procedure SetWaveName(Value : string);
      function GetWaveName : string;
      procedure SetResourceType(Value : string);
      function GetResourceType : string;
    protected
      constructor create(aOwner : TComponent); override;
      destructor Destroy; override;
    public
      function Execute : boolean;
      function StopSound : boolean;
    published
      property WhereWave : TWhereWave read FWhereWave write SetWhereWave;
      property WaveName : string read GetWaveName write SetWaveName;

      {not used in Win32 - always must be 'WAVE'. Leaved for compatibility}
      property ResourceType : string read GetResourceType write SetResourceType;

      property Options : TWaveOptions read FPalyOptions write FPalyOptions default [woSync, woNoDefault, woNoStop];
  End;

Procedure Register;

Implementation

Uses mmsystem;

Constructor TWave.Create(aOwner : TComponent);
Begin
  Inherited Create(aOwner);
  Options := [woSync, woNoDefault, woNoStop];
  GetMem(FWaveName, 256);
  GetMem(FResourceType, 256);
  WaveName := '';
  ResourceType := 'WAVE';
End;

destructor TWave.Destroy;
begin
  FreeMem(FResourceType, 256);
  FreeMem(FWaveName, 256);
  Inherited Destroy;
end;

Procedure TWave.SetWhereWave(Value : TWhereWave);
Begin
  If Value <> FWhereWave
  Then
    FWhereWave := Value;
End;

Procedure TWave.SetWaveName(Value : String);
Begin
  FWaveName := StrPCopy(FWaveName, Value);
End;

function TWave.GetWaveName : String;
Begin
  Result := StrPas(FWaveName);
End;

Procedure TWave.SetResourceType(Value : String);
Begin
  FResourceType := StrPCopy(FResourceType, Value);
End;

function TWave.GetResourceType : String;
Begin
  Result := StrPas(FResourceType);
End;

Function TWave.Execute : Boolean;
Var
  FindResHandle : THandle;
Begin
  Result := False;
  If WaveOutGetNumDevs > 0
  Then
    Begin
      FOptions := 0;
      If woSync In Options
      Then
        FOptions := FOptions Or SND_SYNC
      Else
        FOptions := FOptions Or SND_ASYNC;
      If woNoDefault In Options
      Then
        FOptions := FOptions Or SND_NODEFAULT;
      If woLoop In Options
      Then
        FOptions := FOptions Or SND_LOOP;
      If woNoStop In Options
      Then
        FOptions := FOptions Or SND_NOSTOP;
      If FWhereWave = wwInFile
      Then
        Result := PlaySound(FWaveName, 0, FOptions)
      Else
        Begin
          FindResHandle := FindResource(HInstance, FWaveName, 'WAVE');
          If FindResHandle <> 0
          Then
            Result := PlaySound(FWaveName, HInstance, FOptions or SND_RESOURCE)
          else
            MessageBox(0, 'WAVE resource not found!', 'Error', mb_ok or mb_iconstop);
        End;
    End;
End;

Function TWave.StopSound : Boolean;
Begin
  Result := PlaySound(#0, 0, 0);
End;

Procedure Register;
Begin
  RegisterComponents('a.lee', [TWave]);
End;

End.
